import rich_click as click
import os
import subprocess
from rich.prompt import Prompt
from clir.command import Command
from clir.command import CommandTable
from clir.utils.config import init_config

@click.group()
def cli():
    pass



@cli.command(help="Save new command 💾")
@click.option('-c', '--command', help="Command to be saved", prompt=True)
@click.option('-d', '--description', help="Description of the command", prompt=True)
@click.option('-t', '--tag', help="Tag to be associated with the command", prompt=True)
def new(command, description, tag):
    init_config()
    new_command = Command(command = command, description = description, tag = tag)
    new_command.save_command()

@cli.command(help="Remove command 👋")
@click.option('-t', '--tag', help="Search by tag")
@click.option('-g', '--grep', help="Search by grep")
def rm(tag: str = "", grep: str = ""):
    init_config()
    table = CommandTable(tag=tag, grep=grep)
    table.remove_command()

@cli.command(help="List commands 📃")
@click.option('-t', '--tag', help="Search by tag")
@click.option('-g', '--grep', help="Search by grep")
def ls(tag: str = "", grep: str = ""):
    init_config()    
    table = CommandTable(tag=tag, grep=grep)
    table.show_table()

@cli.command(help="Run command 🚀")
@click.option('-t', '--tag', help="Search by tag")
@click.option('-g', '--grep', help="Search by grep")
def run(tag: str = "", grep: str = ""):
    init_config()
    table = CommandTable(tag=tag, grep=grep)
    table.run_command()

@cli.command(help="Copy command to clipboard 📋")
@click.option('-t', '--tag', help="Search by tag")
@click.option('-g', '--grep', help="Search by grep")
def cp(tag: str = "", grep: str = ""):
    init_config()
    table = CommandTable(tag=tag, grep=grep)
    table.copy_command()

@cli.command(help="Show tags 🏷️")
@click.option('-g', '--grep', help="Search by grep")
def tags(grep: str = ""):
    init_config()
    table = CommandTable(grep=grep)
    table.show_tags()


import os
import re
import json
import clir.utils.filters as filters
from clir.utils.db import get_tag_id_from_command_id, get_tag_from_tag_id

# Create a function that returns all commands
def get_commands(tag: str = "", grep: str = ""):
    current_commands = ""
    json_file_path = os.path.join(os.path.expanduser('~'), '.clir/commands.json')

    try:
        with open(json_file_path, 'r') as json_file:
            current_commands =  json.load(json_file)
    except FileNotFoundError:
        return []
    
    if tag:
        current_commands = filters.filter_by_tag(commands=current_commands, tag=tag)
    if grep:
        current_commands = filters.filter_by_grep(commands=current_commands, grep=grep)
    
    sorted_commands = dict(sorted(current_commands.items(), key=lambda item: item[1]["tag"]))

    return sorted_commands

def transform_commands_to_json(commands):
    commands_json = {}
    
    for command in commands:
        commands_json[command[3]] = {
            "description": command[4],
            "tag": get_tag_from_tag_id(get_tag_id_from_command_id(command[0])),
            "uid": command[0],
            "creation_date": command[1],
            "last_modif_date": command[2]
        }
    
    return commands_json


def get_user_input(arg):
    return input(f"Enter value for '{arg}': ")

def replace_arguments(command):
    # Use regex to find all arguments with underscores
    matches = re.findall(r'_\w+', command)

    # Check that all arguments are unique
    if len(matches) != len(set(matches)):
        print("[bold red]Make sure that all arguments are unique[/bold red]")
        return None
    
    # Prompt the user for values for each argument
    replacements = {arg: get_user_input(arg) for arg in matches}
    
    # Split the command into a list
    command_list = command.split(" ")

    # Replace arguments in the command
    for arg, value in replacements.items():
        for indx,term in enumerate(command_list):
            if arg == term:
                command_list[indx] = value
    
    return " ".join(command_list)

def uuid_to_base64(uuid_obj):
    uuid_bytes = uuid_obj.bytes
    base64_uuid = base64.urlsafe_b64encode(uuid_bytes).rstrip(b'=').decode('ascii')
    return base64_uuid

def base64_to_uuid(base64_uuid):
    padding = '=' * (4 - len(base64_uuid) % 4)  # Add padding if necessary
    uuid_bytes = base64.urlsafe_b64decode(base64_uuid + padding)
    return uuid.UUID(bytes=uuid_bytes)



import sqlite3
import uuid
import base64
from pathlib import Path
import datetime
import shutil
import importlib.resources

# Specify the directory and file name
schema_directory = "clir/config/db/"
schema_file_name = "schema.sql"
db_file_name = "clir.db"
sql_schema_path = Path(schema_directory) / schema_file_name
db_user_version = 1

env_path = Path('~').expanduser() / Path(".clir")
db_file = Path(env_path) / db_file_name

def _verify_tag_exists(tag: str):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    # Check if tag exists
    cursor.execute("SELECT * FROM tags WHERE tag = ?", (tag,))
    tag_exists = cursor.fetchone()

    if not tag_exists:
        tag_uuid = str(uuid.uuid4())

        # Insert a new tag into the database
        insert_tag(tag = tag, tag_uuid = tag_uuid)
    else:
        tag_uuid = cursor.execute("SELECT id FROM tags WHERE tag = ?", (tag,))
        tag_uuid = cursor.fetchone()[0]
    
    # Close the cursor and connection
    cursor.close()
    conn.close()

    return tag_uuid

def _verify_command_exists(command: str):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    # Check if command exists
    cursor.execute("SELECT * FROM commands WHERE command = ?", (command,))
    command_exists = cursor.fetchone()

    if command_exists:
        command_uuid = cursor.execute("SELECT id FROM commands WHERE command = ?", (command,))
    else:
        return False

    # Close the cursor and connection
    cursor.close()
    conn.close()

    return command_uuid

# Creates the DB from the schema file
def create_database(database_name = db_file, schema_file = sql_schema_path):
    # Connect to the SQLite database (or create it if it doesn't exist)
    conn = sqlite3.connect(database_name)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    # Read and execute SQL statements from the schema file
    with importlib.resources.open_text('clir.config.db', 'schema.sql') as f:
        schema = f.read()
        cursor.executescript(schema)
        
    # Commit the changes
    conn.commit()

    # Close the cursor and connection
    cursor.close()
    conn.close()

# Insert a new command into the database
def insert_command_db(command: str, description: str, tag: str = ""):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    command_uuid = str(uuid.uuid4())
    tag_uuid = ""

    creation_date = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')[:-3]
    last_modif_date = creation_date

    tag_uuid = _verify_tag_exists(tag)

    command_exists = _verify_command_exists(command)

    if command_exists:
        modify_command(command, description, tag)
    else:
        # Insert a new command into the database
        cursor.execute("INSERT INTO commands (command, description, id, creation_date, last_modif_date) VALUES (?, ?, ?, ?, ?)", (command, description, command_uuid, creation_date, last_modif_date))

        # Insert a new command into the commands_tags table
        cursor.execute("INSERT INTO commands_tags (command_id, tag_id) VALUES (?, ?)", (command_uuid, tag_uuid))

    # Commit the changes
    conn.commit()

    # Close the cursor and connection
    cursor.close()
    conn.close()

    return 0

# Modify a command in the database
def modify_command(command: str, description: str, tag: str = ""):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    command_uuid = cursor.execute("SELECT id FROM commands WHERE command = ?", (command,))
    command_uuid = cursor.fetchone()[0]
    tag_uuid = ""

    last_modif_date = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')[:-3]

    tag_uuid = _verify_tag_exists(tag)

    # Modify the command in the database
    cursor.execute("UPDATE commands SET description = ?, last_modif_date = ? WHERE command = ?", (description, last_modif_date, command))

    # Modify the command in the commands_tags table
    cursor.execute("UPDATE commands_tags SET tag_id = ? WHERE command_id = ?", (tag_uuid, command_uuid))

    # Commit the changes
    conn.commit()

    # Close the cursor and connection
    cursor.close()
    conn.close()

    return 0


# Remove a command from the database
def remove_command_db(uid: str):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    # Remove a command from the database
    cursor.execute("DELETE FROM commands WHERE id = ?", (uid,))
    cursor.execute("DELETE FROM commands_tags WHERE command_id = ?", (uid,))

    # Commit the changes
    conn.commit()

    # Close the cursor and connection
    cursor.close()
    conn.close()

def verify_command_id_exists(uid: str):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    # Check if command exists
    cursor.execute("SELECT * FROM commands WHERE id = ?", (uid,))
    command_exists = cursor.fetchone()

    # Close the cursor and connection
    cursor.close()
    conn.close()

    return True if command_exists else False

def verify_tag_id_exists(uid: str):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    # Check if tag exists
    cursor.execute("SELECT * FROM tags WHERE id = ?", (uid,))
    tag_exists = cursor.fetchone()

    # Close the cursor and connection
    cursor.close()
    conn.close()

    return True if tag_exists else False

def verify_command_id_tag_relation_exists(command_id: str):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    # Check if command_id exists in the commands_tags table
    cursor.execute("SELECT * FROM commands_tags WHERE command_id = ?", (command_id,))
    command_id_exists = cursor.fetchone()

    # Close the cursor and connection
    cursor.close()
    conn.close()

    return True if command_id_exists else False

# Get all commands from the database
def get_commands_db(tag: str = "", grep: str = ""):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    # Get all commands from the database
    if tag:
        command_ids = get_commands_from_tag(tag)
        query = "SELECT * FROM commands WHERE id IN ({})".format(','.join('?' for _ in command_ids))
        cursor.execute(query, command_ids)
    elif grep:
        cursor.execute("SELECT * FROM commands WHERE command LIKE ? OR description LIKE ?", (f"%{grep}%", f"%{grep}%"))
    else:
        cursor.execute("SELECT * FROM commands")

    commands = cursor.fetchall()

    # Close the cursor and connection
    cursor.close()
    conn.close()

    return commands

# Get all tags from the database
def get_tags_db(grep: str = ""):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    # Get all tags from the database
    if grep:
        cursor.execute("SELECT * FROM tags WHERE tag LIKE ?", (f"%{grep}%",))
    else:
        cursor.execute("SELECT * FROM tags")

    tags = cursor.fetchall()

    # Close the cursor and connection
    cursor.close()
    conn.close()

    return tags

# Insert a new tag into the database
def insert_tag(tag: str, tag_uuid: str = str(uuid.uuid4())):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()
    creation_date = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S.%f')[:-3]
    last_modif_date = creation_date

    # Insert a new tag into the database
    cursor.execute("INSERT INTO tags (tag, id, creation_date, last_modif_date) VALUES (?, ?, ?, ?)", (tag, tag_uuid, creation_date, last_modif_date))

    # Commit the changes
    conn.commit()

    # Close the cursor and connection
    cursor.close()
    conn.close()

    return tag_uuid

# Remove a tag from the database
def remove_tag(tag: str):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    # Remove a tag from the database
    cursor.execute("DELETE FROM tags WHERE tag = ?", (tag,))

    # Commit the changes
    conn.commit()

    # Close the cursor and connection
    cursor.close()
    conn.close()

def get_command_id_from_command(command):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    # Get the command_id from the commands table
    cursor.execute("SELECT id FROM commands WHERE command = ?", (command,))
    command_id = cursor.fetchone()[0]

    # Close the cursor and connection
    cursor.close()
    conn.close()

    return command_id

def get_tag_id_from_command_id(command_id):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    # Get the tag_id from the commands_tags table
    cursor.execute("SELECT * FROM commands_tags WHERE command_id = ?", (command_id,))
    tag_id = cursor.fetchone()

    # Close the cursor and connection
    cursor.close()
    conn.close()

    return tag_id

def get_tag_from_tag_id(tag_id):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    query = "SELECT tag FROM tags WHERE id IN ({})".format(','.join('?' for _ in tag_id))
    cursor.execute(query, tag_id)

    tag = cursor.fetchall()[0][0]
    #tags = [row[0] for row in tag]

    # Close the cursor and connection
    cursor.close()
    conn.close()

    return tag

def get_tag_id_from_tag(tag):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)
    tag_id = ""

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    # Get the tag_id from the tags table
    cursor.execute("SELECT id FROM tags WHERE tag = ?", (tag,))
    try :
        tag_id = cursor.fetchone()[0]
    except:
        pass

    # Close the cursor and connection
    cursor.close()
    conn.close()

    return tag_id

def get_command_ids_from_tag_id(tag_id):
    # Connect to the SQLite database
    conn = sqlite3.connect(db_file)

    # Create a cursor object to interact with the database
    cursor = conn.cursor()

    # Get the command_id from the commands_tags table
    cursor.execute("SELECT * FROM commands_tags WHERE tag_id = ?", (tag_id,))
    command_ids = cursor.fetchall()

    # Close the cursor and connection
    cursor.close()
    conn.close()

    return [id[0] for id in command_ids]

def get_commands_from_tag(tag):
    tag_id = get_tag_id_from_tag(tag)
    command_id = get_command_ids_from_tag_id(tag_id)

    return command_id

# TODO: pass a garbag collector to remove tags that are not used after removing or modifying a command

class DbIntegrity:
    def __init__(self):
        self.commands_ids = ""
        self.commands = ""
        self.tags_ids = ""
        self.tags = ""
    
    def get_commands_ids(self):
        # Connect to the SQLite database
        conn = sqlite3.connect(db_file)

        # Create a cursor object to interact with the database
        cursor = conn.cursor()

        # Get all commands from the database
        cursor.execute("SELECT id FROM commands")

        self.commands_ids = [command_id[0] for command_id in cursor.fetchall()]

        # Close the cursor and connection
        cursor.close()
        conn.close()

        return self.commands_ids
    
    def get_commands(self):
        # Connect to the SQLite database
        conn = sqlite3.connect(db_file)

        # Create a cursor object to interact with the database
        cursor = conn.cursor()

        # Get all commands from the database
        cursor.execute("SELECT command FROM commands")

        self.commands = [command[0] for command in cursor.fetchall()]

        # Close the cursor and connection
        cursor.close()
        conn.close()

        return self.commands

    def get_tags_ids(self):
        # Connect to the SQLite database
        conn = sqlite3.connect(db_file)

        # Create a cursor object to interact with the database
        cursor = conn.cursor()

        # Get all tags from the database
        cursor.execute("SELECT id FROM tags")

        self.tags_ids = [tag_id[0] for tag_id in cursor.fetchall()]

        # Close the cursor and connection
        cursor.close()
        conn.close()

        return self.tags_ids
    
    def get_tags(self):
        # Connect to the SQLite database
        conn = sqlite3.connect(db_file)

        # Create a cursor object to interact with the database
        cursor = conn.cursor()

        # Get all tags from the database
        cursor.execute("SELECT tag FROM tags")

        self.tags = [tag[0] for tag in cursor.fetchall()]

        # Close the cursor and connection
        cursor.close()
        conn.close()

        return self.tags
    
    def get_commands_ids_relation(self):
        # Connect to the SQLite database
        conn = sqlite3.connect(db_file)

        # Create a cursor object to interact with the database
        cursor = conn.cursor()

        # Get all tags from the database
        cursor.execute("SELECT command_id FROM commands_tags")

        command_tag_relation = [command_id[0] for command_id in cursor.fetchall()]

        # Close the cursor and connection
        cursor.close()
        conn.close()

        return command_tag_relation

    def get_tags_ids_relation(self):
        # Connect to the SQLite database
        conn = sqlite3.connect(db_file)

        # Create a cursor object to interact with the database
        cursor = conn.cursor()

        # Get all tags from the database
        cursor.execute("SELECT tag_id FROM commands_tags")

        tag_command_relation = [tag_id[0] for tag_id in cursor.fetchall()]

        # Close the cursor and connection
        cursor.close()
        conn.close()

        return tag_command_relation

    def main(self):
        return self.get_commands_ids(), self.get_commands(), self.get_tags_ids(), self.get_tags(), self.get_commands_ids_relation(), self.get_tags_ids_relation()


import os
import shutil
import subprocess
from pathlib import Path
from clir.utils.db import create_database
from clir.utils.core import get_commands
from clir.utils.db import insert_command_db
import importlib.resources

config_directory = "clir/config/"
env_path = Path('~').expanduser() / Path(".clir")

def check_config():
    dir_path = os.path.join(os.path.expanduser('~'), '.clir')
    db_file_path = os.path.join(dir_path, 'clir.db')
    config_file_path = os.path.join(dir_path, 'clir.conf')

    return os.path.exists(db_file_path) and os.path.exists(config_file_path)

def back_json_commands():
    source_file = f"{env_path}/commands.json"
    destination_file = f"{env_path}/commands.json.backup"

    shutil.copyfile(source_file, destination_file)

    print(f"Backup of json commands stored in {source_file} to {destination_file} complete.")

def migrate_json_to_sqlite():
    if os.path.exists(f"{env_path}/commands.json"):
        back_json_commands()
        print("Migrating json stored commands to sqlite database...")
        
        commands = get_commands()

        for command, data in commands.items():
            print(f"Inserting command: {command}")
            print(f"Description: {data['description']}")
            print(f"Tag: {data['tag']}")
            insert_command_db(command, data['description'], data['tag'])
            print("---")

        os.remove(f"{env_path}/commands.json")
        print("Migration complete")


def create_config_files():
    dir_path = os.path.join(os.path.expanduser('~'), '.clir')
    os.makedirs(dir_path, exist_ok=True)
    
    # Define the file path and name
    files = ['commands.json']

    # Check if the file already exists
    for file in files:
        file_path = os.path.join(dir_path, file)
        if not os.path.exists(file_path):
            # Create the file
            with open(file_path, 'w') as file_object:
                file_object.write('{}')

            print(f'File "{file_path}" created successfully.')
        else:
            print(f'A clir environment already exists in "{dir_path}".')

def copy_config_files():
    dir_path = os.path.join(os.path.expanduser('~'), '.clir')
    os.makedirs(dir_path, exist_ok=True)
    
    # Define the file path and name
    files = ['clir.conf']

    # Check if the file already exists
    for file in files:
        with importlib.resources.open_text('clir.config', 'clir.conf') as f:
            conf = f.read()
            with open(f"{env_path}/{file}", "w") as file:
                file.write(conf)

        print(f"Copying {file} file to {env_path}")

def init_config():
    if not check_config():
        dir_path = os.path.join(os.path.expanduser('~'), '.clir')
        os.makedirs(dir_path, exist_ok=True)

        print("Setting up initial configuration")
        print("Creating database...")
        create_database()
        print("Database created successfully")

        print("Creating config files...")
        create_config_files()

        print("Copying other config files...")
        copy_config_files()

        migrate_json_to_sqlite()
    
    if not check_config():
        print("Could not set the initial configuration Up.")
        return
    

def verify_xclip_installation(package: str = ""):
    if package == "xclip":
        try:
            subprocess.run(["xclip", "-version"], check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            return True
        except:
            return False
    if package == "pbcopy":
        try:
            subprocess.run(["which", "pbcopy"], check=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            return True
        except:
            return False
    
    return "No package specified"


import os
import re
import json
import uuid
import platform
import subprocess
from rich import box
from rich import print
from rich.console import Console
from rich.table import Table
from rich.prompt import Prompt
from clir.utils.config import verify_xclip_installation
from clir.utils.core import get_commands, replace_arguments, transform_commands_to_json
from clir.utils.db import insert_command_db, get_commands_db, remove_command_db, verify_command_id_exists, verify_command_id_tag_relation_exists

class Command:
    def __init__(self, command: str = "", description: str = "", tag: str = ""):
        self.command = command
        self.description = description
        self.tag = tag
        self.uid = uuid.uuid4()

    def __str__(self):
        return f"{self.command} {self.description} {self.tag}"
    
    def __repr__(self):
        return f"{self.command} {self.description} {self.tag}"

    def save_command(self):
        current_commands = get_commands_db()

        command = self.command
        desc = self.description
        tag = self.tag

        if not tag:
            tag = "clir"

        insert_command_db(command, desc, tag)

        print(f'Command saved successfuly')


#Create class Table
class CommandTable:
    def __init__(self, tag: str = "", grep: str = ""):
        self.commands = transform_commands_to_json(get_commands_db(tag = tag, grep = grep))
        self.tag = tag
        self.grep = grep

    def __str__(self):
        return f"{self.command} {self.description} {self.tag}"

    def __repr__(self):
        return f"{self.command} {self.description} {self.tag}"

    def show_table(self):

        commands = self.commands
        
        table = Table(show_lines=True, box=box.ROUNDED, style="grey46")
        table.add_column("ID 📇", style="white bold", no_wrap=True)
        table.add_column("Command 💻", style="green bold", no_wrap=True)
        table.add_column("Description 📕", style="magenta bold")
        table.add_column("Tag 🏷️", style="cyan bold")
        
        for indx, command in enumerate(commands):
            desc_len = 50
            command_len = 50
            description = commands[command]["description"]
            split_description = "\n".join([description[i:i+desc_len] for i in range(0, len(description), desc_len)])
            split_command = []
            local_command = ""
            for command_term in command.split(" "):
                if len(local_command) + len(command_term) > command_len:
                    split_command.append(local_command)
                    local_command = command_term + " "
                else:
                    local_command += command_term + " "
                
            split_command.append(local_command)
            
            display_command = " \ \n".join(split_command)
            table.add_row(str(indx+1), display_command, split_description, commands[command]["tag"])

        console = Console()
        console.print(table)
        print(f"Showing {str(len(commands))} commands")
    
    def run_command(self):
        current_commands = self.commands

        uid = self.get_command_uid()
        
        command = ""
        for c in current_commands:
            if current_commands[c]["uid"] == uid:
                command = c
        
        command = replace_arguments(command)
        if uid and command:
            print(f'[bold green]Running command:[/bold green] {command}')
            os.system(command)
            subprocess.Popen(['bash', '-ic', 'set -o history; history -s "$1"', '_', command])
    
    def copy_command(self):
        current_commands = self.commands

        uid = self.get_command_uid()
        
        command = ""
        for c in current_commands:
            if current_commands[c]["uid"] == uid:
                command = c
        
        if uid:
            print(f'Copying command: {command}')
            if platform.system() == "Darwin":
                # Verify that pbcopy is installed
                if verify_xclip_installation(package = "pbcopy"):
                    os.system(f'printf "{command}" | pbcopy')
                else:
                    print("pbcopy is not installed, this command needs pbcopy to work properly")
                    return
            elif platform.system() == "Linux":
                # Verify that xclip is installed
                if verify_xclip_installation(package = "xclip"):
                    os.system(f'echo -n "{command}" | xclip -selection clipboard')
                else:
                    print("xclip is not installed, this command needs xclip to work properly")
                    return
            else:
                print("OS not supported")
    
    def show_tags(self):
        current_commands = self.commands

        tags = []
        for command in current_commands:
            tags.append(current_commands[command]["tag"])
        
        tags = list(dict.fromkeys(tags))

        table = Table(show_lines=True, box=box.ROUNDED, style="grey46")
        table.add_column("Tags 🏷️", style="cyan bold")
        
        for tag in tags:
            table.add_row(tag)

        console = Console()
        console.print(table)
        print(f"Showing {str(len(tags))} tags")
    

    # Create a function that deletes a command when passing its uid
    def remove_command(self):

        uid = self.get_command_uid()

        remove_command_db(uid)
        verify_command_id_exists(uid)
        verify_command_id_tag_relation_exists(uid)

        if verify_command_id_exists(uid):
            print(f'Command not removed')
        elif not verify_command_id_exists(uid) and verify_command_id_tag_relation_exists(uid):
            print(f'Command removed successfuly but relation to tag not removed')
        elif not verify_command_id_exists(uid) and not verify_command_id_tag_relation_exists(uid):
            print(f'Command removed successfuly')


    def get_command_uid(self):
        self.show_table()
        command_id = Prompt.ask("Enter command ID")

        try:
            while int(command_id) > len(self.commands) or int(command_id) < 1:
                print("ID not valid")
                command_id = Prompt.ask("Enter command ID")
            
            command = self.commands[list(self.commands.keys())[int(command_id)-1]]["uid"]

            return command
        except ValueError:
            print("ID must be an integer")
        
        return ""


import re
import clir.utils.core as core

def filter_by_tag(commands: dict = {}, tag: str = ""):
    if commands:
        current_commands = commands
    else:
        current_commands = core.get_commands()

    tag_commands = {}
    for command in current_commands:
        if current_commands[command]["tag"] == tag:
            tag_commands[command] = current_commands[command]

    return tag_commands

def filter_by_grep(commands: dict = {}, grep: str = ""):
    if commands:
        current_commands = commands
    else:
        current_commands = core.get_commands()

    grep_commands = {}
    pattern = grep
    for command in current_commands:
        text = command + " " + current_commands[command]["description"]# + " " + current_commands[command]["tag"]
        match = re.findall(pattern, text, re.IGNORECASE)
        if match:
            grep_commands[command] = current_commands[command]

    return grep_commands